#include "include/schema.include"
#include "include/sequence.include"
#include "include/constraint.include"
#include "include/type.include"

-- 5.6.0
#ifdef H2
-- Make sure the schema exists in the H2 demo database
CREATE USER IF NOT EXISTS X4S PASSWORD 'x4' ADMIN;
CREATE ROLE IF NOT EXISTS X4_SCHEMA;
CREATE SCHEMA IF NOT EXISTS X4_SCHEMA AUTHORIZATION SA;
GRANT X4_SCHEMA TO X4S;
#endif

CREATE TABLE X4_SCHEMA.X4_VERSION
(
  VERSION X4_TYPE_VARCHAR(128),
  INSTALL_DATE X4_TYPE_DATETIME_NULLABLE DEFAULT NOW,
  UQ(X4_VERSION_VID, VERSION, INSTALL_DATE)
);

CREATE TABLE X4_SCHEMA.X4_CASES
(
  ID X4_TYPE_ID,
  DOMAIN_ID X4_TYPE_VARCHAR(256),
  TYPE X4_TYPE_VARCHAR(256),
  STATE X4_TYPE_VARCHAR(256),
  CREATOR X4_TYPE_VARCHAR(128),
  CREATED X4_TYPE_DATETIME,
  UPDATED X4_TYPE_DATETIME,
  AGENT X4_TYPE_VARCHAR(128),
  AGENT_GROUP X4_TYPE_VARCHAR(128),
  REMINDER X4_TYPE_DATETIME,
  DEADLINE X4_TYPE_DATETIME,
  PRIO X4_TYPE_INTEGER,
  PARENT_ID X4_TYPE_FK_NULLABLE,
  PK(X4_CASES_ID, ID),
  UQ(X4_CASES_DOMAIN_ID, DOMAIN_ID)
);

ALTER_TABLE_FK(X4_CASES_PARENT_ID, X4_CASES, PARENT_ID, X4_CASES, ID, X4_SCHEMA);
SEQ(ID,X4_CASES,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.X4_TASKS
(
  ID X4_TYPE_ID,
  DOMAIN_ID X4_TYPE_VARCHAR(256),
  TYPE X4_TYPE_VARCHAR(256),
  STATE X4_TYPE_VARCHAR(256),
  CREATOR X4_TYPE_VARCHAR(128),
  CREATED X4_TYPE_DATETIME,
  UPDATED X4_TYPE_DATETIME,
  AGENT X4_TYPE_VARCHAR(128),
  AGENT_GROUP X4_TYPE_VARCHAR(128),
  REMINDER X4_TYPE_DATETIME,
  DEADLINE X4_TYPE_DATETIME,
  PRIO X4_TYPE_INTEGER,
  PARENT_ID X4_TYPE_FK_NULLABLE,
  CASE_ID X4_TYPE_FK,
  PK(X4_TASKS_ID, ID),
  UQ(X4_TASKS_DOMAIN_ID, DOMAIN_ID),
  FK(X4_TASKS_CASE_ID, CASE_ID, X4_CASES, ID, X4_SCHEMA)
);

ALTER_TABLE_FK(X4_TASKS_PARENT_ID, X4_TASKS, PARENT_ID, X4_TASKS, ID, X4_SCHEMA);
SEQ(ID,X4_TASKS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.X4_COMMENTS
(
  ID X4_TYPE_ID,
  CREATOR X4_TYPE_VARCHAR(128),
  CREATED X4_TYPE_DATETIME,
  MESSAGE X4_TYPE_VARCHAR(1024),
  CASE_ID X4_TYPE_FK_NULLABLE,
  TASK_ID X4_TYPE_FK_NULLABLE,
  PK(X4_COMMENTS_ID, ID),
  FK(X4_COMMENTS_CASE_ID, CASE_ID, X4_CASES, ID, X4_SCHEMA),
  FK(X4_COMMENTS_TASK_ID, TASK_ID, X4_TASKS, ID, X4_SCHEMA),
  CK(X4_COMMENTS, CASE_ID IS NOT NULL OR TASK_ID IS NOT NULL)
);

SEQ(ID,X4_COMMENTS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.X4_JOURNALS
(
  ID X4_TYPE_ID,
  CREATOR X4_TYPE_VARCHAR(128),
  CREATED X4_TYPE_DATETIME,
  MESSAGE X4_TYPE_VARCHAR(1024),
  VALUE_NAME X4_TYPE_VARCHAR(128),
  OLD_VALUE X4_TYPE_VARCHAR(128),
  NEW_VALUE X4_TYPE_VARCHAR(128),
  CASE_ID X4_TYPE_FK_NULLABLE,
  TASK_ID X4_TYPE_FK_NULLABLE,
  PK(X4_JOURNALS_ID, ID),
  FK(X4_JOURNALS_CASE_ID, CASE_ID, X4_CASES, ID, X4_SCHEMA),
  FK(X4_JOURNALS_TASK_ID, TASK_ID, X4_TASKS, ID, X4_SCHEMA),
  CK(X4_JOURNALS, CASE_ID IS NOT NULL OR TASK_ID IS NOT NULL)
);

SEQ(ID,X4_JOURNALS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.X4_CASES_TO_PID
(
  PID X4_TYPE_VARCHAR(200),
  CASE_ID X4_TYPE_FK,
  FK(X4_CTP_CASE_ID, CASE_ID, X4_CASES, ID, X4_SCHEMA)
);

CREATE TABLE X4_SCHEMA.X4_TASKS_TO_PID
(
  PID X4_TYPE_VARCHAR(200),
  TASK_ID X4_TYPE_FK,
  FK(X4_CTP_TASK_ID, TASK_ID, X4_TASKS, ID, X4_SCHEMA)
);

CREATE TABLE X4_SCHEMA.X4_DATA_REFERENCES
(
  ID X4_TYPE_ID,
  TYPE X4_TYPE_VARCHAR(128),
  URL X4_TYPE_VARCHAR(2048),
  CASE_ID X4_TYPE_FK_NULLABLE,
  TASK_ID X4_TYPE_FK_NULLABLE,
  PK(X4_DATA_REFERENCES, ID),
  FK(X4_DATA_REFERENCES_CASE_ID, CASE_ID, X4_CASES, ID, X4_SCHEMA),
  FK(X4_DATA_REFERENCES_TASK_ID, TASK_ID, X4_TASKS, ID, X4_SCHEMA),
  CK(X4_DATA_REFERENCES, CASE_ID IS NOT NULL OR TASK_ID IS NOT NULL)
);

SEQ(ID,X4_DATA_REFERENCES,X4_SCHEMA);