#include "include/schema.include"
#include "include/constraint.include"
#include "include/type.include"
#include "include/sequence.include"

-- 5.7.0
-- Create new data table 
CREATE TABLE X4_SCHEMA.X4_DATA
(
  ID X4_TYPE_ID,
  TYPE X4_TYPE_VARCHAR(128),
  DATA X4_TYPE_BLOB,
  CASE_ID X4_TYPE_FK_NULLABLE,
  TASK_ID X4_TYPE_FK_NULLABLE,
  PK(X4_DATA, ID),
  FK(X4_DATA_CASE_ID, CASE_ID, X4_CASES, ID, X4_SCHEMA),
  FK(X4_DATA_TASK_ID, TASK_ID, X4_TASKS, ID, X4_SCHEMA),
  CK(X4_DATA, CASE_ID IS NOT NULL OR TASK_ID IS NOT NULL)
);

SEQ(ID,X4_DATA,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.X4_OBJECT_IDS
(
  ID X4_TYPE_ID,
  TABLE_NAME X4_TYPE_VARCHAR(128),
  PK(X4_OBJECT_IDS, ID)
);

SEQ(ID,X4_OBJECT_IDS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.X4_OBJECT_TO_PID
(
  PID X4_TYPE_VARCHAR(200),
  OID X4_TYPE_FK,
  FK(X4_PID_OID, OID, X4_OBJECT_IDS, ID, X4_SCHEMA)
);

ALTER TABLE X4_SCHEMA.X4_COMMENTS ADD OID X4_TYPE_FK;
ALTER TABLE X4_SCHEMA.X4_JOURNALS ADD OID X4_TYPE_FK;
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES ADD OID X4_TYPE_FK;
ALTER TABLE X4_SCHEMA.X4_DATA ADD OID X4_TYPE_FK;

ALTER_TABLE_FK(X4_COMMENTS_OID, X4_COMMENTS, OID, X4_OBJECT_IDS, ID, X4_SCHEMA);
CREATE INDEX IDX_X4_COMMENTS_OID ON X4_SCHEMA.X4_COMMENTS (OID);
ALTER_TABLE_FK(X4_JOURNALS_OID, X4_JOURNALS, OID, X4_OBJECT_IDS, ID, X4_SCHEMA);
CREATE INDEX IDX_X4_JOURNALS_OID ON X4_SCHEMA.X4_JOURNALS (OID);
ALTER_TABLE_FK(X4_DATA_REFERENCES_OID, X4_DATA_REFERENCES, OID, X4_OBJECT_IDS, ID, X4_SCHEMA);
CREATE INDEX IDX_X4_DATA_REFERENCES_OID ON X4_SCHEMA.X4_DATA_REFERENCES (OID);
ALTER_TABLE_FK(X4_DATA_OID, X4_DATA, OID, X4_OBJECT_IDS, ID, X4_SCHEMA);
CREATE INDEX IDX_X4_DATA_OID ON X4_SCHEMA.X4_DATA (OID);

ALTER TABLE X4_SCHEMA.X4_COMMENTS DROP CONSTRAINT FK_X4_COMMENTS_CASE_ID;
ALTER TABLE X4_SCHEMA.X4_COMMENTS DROP CONSTRAINT FK_X4_COMMENTS_TASK_ID;
ALTER TABLE X4_SCHEMA.X4_COMMENTS DROP CONSTRAINT CHK_X4_COMMENTS;
ALTER TABLE X4_SCHEMA.X4_COMMENTS DROP COLUMN CASE_ID;
ALTER TABLE X4_SCHEMA.X4_COMMENTS DROP COLUMN TASK_ID;

ALTER TABLE X4_SCHEMA.X4_JOURNALS DROP CONSTRAINT FK_X4_JOURNALS_CASE_ID;
ALTER TABLE X4_SCHEMA.X4_JOURNALS DROP CONSTRAINT FK_X4_JOURNALS_TASK_ID;
ALTER TABLE X4_SCHEMA.X4_JOURNALS DROP CONSTRAINT CHK_X4_JOURNALS;
ALTER TABLE X4_SCHEMA.X4_JOURNALS DROP COLUMN CASE_ID;
ALTER TABLE X4_SCHEMA.X4_JOURNALS DROP COLUMN TASK_ID;

ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES DROP CONSTRAINT FK_X4_DATA_REFERENCES_CASE_ID;
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES DROP CONSTRAINT FK_X4_DATA_REFERENCES_TASK_ID;
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES DROP CONSTRAINT CHK_X4_DATA_REFERENCES;
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES DROP COLUMN CASE_ID;
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES DROP COLUMN TASK_ID;

ALTER TABLE X4_SCHEMA.X4_DATA DROP CONSTRAINT FK_X4_DATA_CASE_ID;
ALTER TABLE X4_SCHEMA.X4_DATA DROP CONSTRAINT FK_X4_DATA_TASK_ID;
ALTER TABLE X4_SCHEMA.X4_DATA DROP CONSTRAINT CHK_X4_DATA;
ALTER TABLE X4_SCHEMA.X4_DATA DROP COLUMN CASE_ID;
ALTER TABLE X4_SCHEMA.X4_DATA DROP COLUMN TASK_ID;

DROP TABLE X4_SCHEMA.X4_CASES_TO_PID;
DROP TABLE X4_SCHEMA.X4_TASKS_TO_PID;

ALTER TABLE X4_SCHEMA.X4_TASKS DROP CONSTRAINT FK_X4_TASKS_CASE_ID;
ALTER TABLE X4_SCHEMA.X4_CASES DROP CONSTRAINT FK_X4_CASES_PARENT_ID;
ALTER TABLE X4_SCHEMA.X4_CASES DROP CONSTRAINT PK_X4_CASES_ID;

ALTER TABLE X4_SCHEMA.X4_TASKS DROP CONSTRAINT FK_X4_TASKS_PARENT_ID;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP CONSTRAINT PK_X4_TASKS_ID;

DROP_SEQ(ID,X4_CASES,X4_SCHEMA);
DROP_SEQ(ID,X4_TASKS,X4_SCHEMA);

ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN ID;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN ID;

ALTER TABLE X4_SCHEMA.X4_CASES ADD ID X4_TYPE_FK;
ALTER TABLE X4_SCHEMA.X4_TASKS ADD ID X4_TYPE_FK;

ALTER_TABLE_FK(X4_CASES_OID, X4_CASES, ID, X4_OBJECT_IDS, ID, X4_SCHEMA);
ALTER_TABLE_FK(X4_TASKS_OID, X4_TASKS, ID, X4_OBJECT_IDS, ID, X4_SCHEMA);

ALTER TABLE X4_SCHEMA.X4_CASES ADD PK(X4_CASES_ID, ID);
ALTER_TABLE_FK(X4_CASES_PARENT_ID, X4_CASES, PARENT_ID, X4_CASES, ID, X4_SCHEMA);
ALTER_TABLE_FK(X4_TASKS_CASE_ID, X4_TASKS, CASE_ID, X4_CASES, ID, X4_SCHEMA);

ALTER TABLE X4_SCHEMA.X4_TASKS ADD PK(X4_TASKS_ID, ID);
ALTER_TABLE_FK(X4_TASKS_PARENT_ID, X4_TASKS, PARENT_ID, X4_TASKS, ID, X4_SCHEMA);

CREATE INDEX IDX_X4_TASKS_CASE_ID ON X4_SCHEMA.X4_TASKS (CASE_ID);

ALTER TABLE X4_SCHEMA.X4_DATA ADD CHAR_SET X4_TYPE_VARCHAR(50);
ALTER TABLE X4_SCHEMA.X4_DATA ADD MIME_TYPE X4_TYPE_VARCHAR(50);
ALTER TABLE X4_SCHEMA.X4_DATA ADD DESCRIPTION X4_TYPE_VARCHAR_NULLABLE(200);
ALTER TABLE X4_SCHEMA.X4_DATA_REFERENCES ADD DESCRIPTION X4_TYPE_VARCHAR_NULLABLE(10);