#include "include/schema.include"
#include "include/constraint.include"
#include "include/type.include"
#include "include/sequence.include"

--5.8.0

#ifdef ORACLE
	#define ALTER_COLUMN MODIFY
#else
	#define ALTER_COLUMN ALTER COLUMN
#endif

#ifdef POSTGRESQL
	#define ALTER_COLUMN_TYPE TYPE
#else
	#define ALTER_COLUMN_TYPE
#endif

#ifndef MSSQL
	#define GO
#endif

#if defined POSTGRESQL | H2
	#ifdef POSTGRESQL
		#define ALTER_COLUMN_SET_NULL DROP NOT NULL
	#else
		#define ALTER_COLUMN_SET_NULL SET NULL
	#endif
ALTER TABLE X4_SCHEMA.X4_CASES ALTER_COLUMN REMINDER ALTER_COLUMN_SET_NULL;
ALTER TABLE X4_SCHEMA.X4_CASES ALTER_COLUMN DEADLINE ALTER_COLUMN_SET_NULL;
ALTER TABLE X4_SCHEMA.X4_CASES ALTER_COLUMN AGENT ALTER_COLUMN_SET_NULL;
ALTER TABLE X4_SCHEMA.X4_CASES ALTER_COLUMN AGENT_GROUP ALTER_COLUMN_SET_NULL;
ALTER TABLE X4_SCHEMA.X4_TASKS ALTER_COLUMN REMINDER ALTER_COLUMN_SET_NULL;
ALTER TABLE X4_SCHEMA.X4_TASKS ALTER_COLUMN DEADLINE ALTER_COLUMN_SET_NULL;
ALTER TABLE X4_SCHEMA.X4_TASKS ALTER_COLUMN AGENT ALTER_COLUMN_SET_NULL;
ALTER TABLE X4_SCHEMA.X4_TASKS ALTER_COLUMN AGENT_GROUP ALTER_COLUMN_SET_NULL;
ALTER TABLE X4_SCHEMA.X4_JOURNALS ALTER_COLUMN OLD_VALUE ALTER_COLUMN_SET_NULL;
ALTER TABLE X4_SCHEMA.X4_JOURNALS ALTER_COLUMN NEW_VALUE ALTER_COLUMN_SET_NULL;
#endif
ALTER TABLE X4_SCHEMA.X4_CASES ALTER_COLUMN REMINDER ALTER_COLUMN_TYPE X4_TYPE_DATETIME_NULLABLE;
ALTER TABLE X4_SCHEMA.X4_CASES ALTER_COLUMN DEADLINE ALTER_COLUMN_TYPE X4_TYPE_DATETIME_NULLABLE;
ALTER TABLE X4_SCHEMA.X4_CASES ALTER_COLUMN AGENT ALTER_COLUMN_TYPE X4_TYPE_VARCHAR_NULLABLE(100);
ALTER TABLE X4_SCHEMA.X4_CASES ALTER_COLUMN AGENT_GROUP ALTER_COLUMN_TYPE X4_TYPE_VARCHAR_NULLABLE(100);
ALTER TABLE X4_SCHEMA.X4_TASKS ALTER_COLUMN REMINDER ALTER_COLUMN_TYPE X4_TYPE_DATETIME_NULLABLE;
ALTER TABLE X4_SCHEMA.X4_TASKS ALTER_COLUMN DEADLINE ALTER_COLUMN_TYPE X4_TYPE_DATETIME_NULLABLE;
ALTER TABLE X4_SCHEMA.X4_TASKS ALTER_COLUMN AGENT ALTER_COLUMN_TYPE X4_TYPE_VARCHAR_NULLABLE(100);
ALTER TABLE X4_SCHEMA.X4_TASKS ALTER_COLUMN AGENT_GROUP ALTER_COLUMN_TYPE X4_TYPE_VARCHAR_NULLABLE(100);
ALTER TABLE X4_SCHEMA.X4_JOURNALS ALTER_COLUMN OLD_VALUE ALTER_COLUMN_TYPE X4_TYPE_VARCHAR_NULLABLE(200);
ALTER TABLE X4_SCHEMA.X4_JOURNALS ALTER_COLUMN NEW_VALUE ALTER_COLUMN_TYPE X4_TYPE_VARCHAR_NULLABLE(200);

ALTER TABLE X4_SCHEMA.X4_CASES ADD CK(CASES_ASSIGNMENT, AGENT IS NOT NULL OR AGENT_GROUP IS NOT NULL);
ALTER TABLE X4_SCHEMA.X4_TASKS ADD CK(TASKS_ASSIGNMENT, AGENT IS NOT NULL OR AGENT_GROUP IS NOT NULL);

ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS ADD LOCKED X4_TYPE_DATETIME_NULLABLE;
ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS ADD LOCKED_BY X4_TYPE_VARCHAR_NULLABLE(50);
ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS ADD OBJECT_TYPE X4_TYPE_VARCHAR_NULLABLE(50) DEFAULT 'Unknown' NN;
ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS ADD CREATOR X4_TYPE_VARCHAR_NULLABLE(100) DEFAULT 'Unknown' NN;
ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS ADD CREATED X4_TYPE_DATETIME_NULLABLE DEFAULT NOW NN;
GO
#ifndef H2
UPDATE X4_SCHEMA.X4_OBJECT_IDS SET OBJECT_TYPE = TABLE_NAME;
	#ifdef ORACLE
		UPDATE X4_SCHEMA.X4_OBJECT_IDS SET (CREATOR, CREATED) = (SELECT X4_CASES.CREATOR, X4_CASES.CREATED FROM X4_SCHEMA.X4_CASES WHERE X4_OBJECT_IDS.ID = X4_CASES.ID);
		UPDATE X4_SCHEMA.X4_OBJECT_IDS SET (CREATOR, CREATED) = (SELECT X4_TASKS.CREATOR, X4_TASKS.CREATED FROM X4_SCHEMA.X4_TASKS WHERE X4_OBJECT_IDS.ID = X4_TASKS.ID);
	#else
		UPDATE X4_SCHEMA.X4_OBJECT_IDS SET CREATOR = X4_CASES.CREATOR, CREATED = X4_CASES.CREATED FROM X4_SCHEMA.X4_CASES WHERE X4_OBJECT_IDS.ID = X4_CASES.ID;
		UPDATE X4_SCHEMA.X4_OBJECT_IDS SET CREATOR = X4_TASKS.CREATOR, CREATED = X4_TASKS.CREATED FROM X4_SCHEMA.X4_TASKS WHERE X4_OBJECT_IDS.ID = X4_TASKS.ID;
	#endif
#endif
ALTER TABLE X4_SCHEMA.X4_OBJECT_IDS DROP COLUMN TABLE_NAME;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN CREATOR;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN CREATED;
ALTER TABLE X4_SCHEMA.X4_CASES DROP COLUMN UPDATED;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN CREATOR;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN CREATED;
ALTER TABLE X4_SCHEMA.X4_TASKS DROP COLUMN UPDATED;

CREATE TABLE X4_SCHEMA.X4_CASE_DEFINITIONS
(
ID X4_TYPE_ID,
NAME X4_TYPE_VARCHAR(50),
URL X4_TYPE_VARCHAR(1000),
PK(X4_CASE_DEF_ID, ID),
UQ(X4_CASE_DEF_URL, URL)
);

SEQ(ID,X4_CASE_DEFINITIONS,X4_SCHEMA);

ALTER TABLE X4_SCHEMA.X4_CASES ADD CASE_DEFINITION_ID X4_TYPE_FK_NULLABLE;
ALTER TABLE X4_SCHEMA.X4_CASES ADD FK(X4_CASE_DEF, CASE_DEFINITION_ID, X4_CASE_DEFINITIONS, ID, X4_SCHEMA);

CREATE TABLE X4_SCHEMA.X4_CATEGORY_DEFINITIONS
(
ID X4_TYPE_ID,
NAME X4_TYPE_VARCHAR(50),
DESCRIPTION X4_TYPE_VARCHAR_NULLABLE(200),
ORDER_INDEX X4_TYPE_INTEGER,
CASE_DEFINITION_ID X4_TYPE_FK,
PK(X4_CATEGORY_DEF_ID, ID),
FK(X4_CATEGORY_DEF_CASE_DEF, CASE_DEFINITION_ID, X4_CASE_DEFINITIONS, ID, X4_SCHEMA),
UQ(X4_CATEGORY_DEFINITIONS, NAME, CASE_DEFINITION_ID)
);

SEQ(ID,X4_CATEGORY_DEFINITIONS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.X4_GROUP_DEFINITIONS
(
ID X4_TYPE_ID,
NAME X4_TYPE_VARCHAR(50),
DESCRIPTION X4_TYPE_VARCHAR_NULLABLE(200),
ORDER_INDEX X4_TYPE_INTEGER,
CATEGORY_ID X4_TYPE_FK,
PK(X4_GROUP_DEF_ID, ID),
FK(X4_GROUP_CATEGORY_DEF_ID, CATEGORY_ID, X4_CATEGORY_DEFINITIONS, ID, X4_SCHEMA),
UQ(X4_GROUP_DEFINITIONS, NAME, CATEGORY_ID)
);

SEQ(ID,X4_GROUP_DEFINITIONS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.X4_FEATURE_DEFINITIONS
(
ID X4_TYPE_ID,
NAME X4_TYPE_VARCHAR(50),
DESCRIPTION X4_TYPE_VARCHAR_NULLABLE(200),
DATATYPE X4_TYPE_VARCHAR(20),
DEFAULT_VALUE_STRING X4_TYPE_VARCHAR_NULLABLE(200),
DEFAULT_VALUE_INTEGER X4_TYPE_INTEGER_NULLABLE,
DEFAULT_VALUE_DECIMAL X4_TYPE_NUMBER_NULLABLE,
DEFAULT_VALUE_BOOLEAN X4_TYPE_BOOLEAN_NULLABLE,
DEFAULT_VALUE_DATETIME X4_TYPE_DATETIME_NULLABLE,
ORDER_INDEX X4_TYPE_INTEGER,
SEARCHABLE X4_TYPE_BOOLEAN,
GROUP_ID X4_TYPE_FK,
CASE_DEFINITION_ID X4_TYPE_FK,
PK(X4_FEATURE_DEF_ID, ID),
FK(X4_FEATURE_DEF_GROUP_ID, GROUP_ID, X4_GROUP_DEFINITIONS, ID, X4_SCHEMA),
FK(X4_FEATURE_DEF_CASE_DEF, CASE_DEFINITION_ID, X4_CASE_DEFINITIONS, ID, X4_SCHEMA),
UQ(X4_FEATURE_DEFINITIONS, NAME, CASE_DEFINITION_ID)
);

SEQ(ID,X4_FEATURE_DEFINITIONS,X4_SCHEMA);

CREATE TABLE X4_SCHEMA.X4_FEATURES
(
	ID X4_TYPE_ID,
	VALUE_STRING X4_TYPE_VARCHAR_NULLABLE(200),
	VALUE_INTEGER X4_TYPE_INTEGER_NULLABLE,
	VALUE_DECIMAL X4_TYPE_NUMBER_NULLABLE,
	VALUE_BOOLEAN X4_TYPE_BOOLEAN_NULLABLE,
	VALUE_DATETIME X4_TYPE_DATETIME_NULLABLE,
	OID X4_TYPE_FK,
	FEATURE_DEF_ID X4_TYPE_FK,
	PK(X4_FEATURES, ID),
	FK(X4_FEATURES_OID, OID, X4_OBJECT_IDS, ID, X4_SCHEMA),
	FK(X4_FEATURE_DEF_ID, FEATURE_DEF_ID, X4_FEATURE_DEFINITIONS, ID, X4_SCHEMA),
	UQ(X4_FEATURES, OID, FEATURE_DEF_ID)
);

SEQ(ID,X4_FEATURES,X4_SCHEMA);