#include "include/schema.include"
#include "include/constraint.include"

#define QUOTE '
#ifdef ORACLE
	#define DROP_CONSTRAINT(table,schema,constraint,statement) \
		DECLARE \
			c INTEGER; \
		BEGIN \
		 	SELECT count(*) into c FROM ALL_CONSTRAINTS WHERE CONSTRAINT_NAME = REPLACE( QUOTE constraint QUOTE, ' ', ''); \
			IF c > 0 THEN \
				EXECUTE IMMEDIATE QUOTE ALTER TABLE schema.table DROP CONSTRAINT constraint QUOTE; \
                EXECUTE IMMEDIATE QUOTE statement QUOTE; \
			END IF; \
		END; \
        __CR__/__CR__ \
        --
#elif POSTGRESQL
	#define DROP_CONSTRAINT(table,schema,constraint,statement) \
        DO $$ \
        BEGIN \
            IF EXISTS ( SELECT constraint_name FROM information_schema.check_constraints \
                        WHERE   constraint_schema = REPLACE(QUOTE schema QUOTE, ' ', '') \
                        AND   constraint_name = REPLACE(QUOTE constraint QUOTE, ' ', '') \
                    ) \
            THEN \
                ALTER TABLE schema.table DROP CONSTRAINT IF EXISTS constraint; \
                statement; \
            END IF; \
        END$$ 
#elif MSSQL
	#define DROP_CONSTRAINT(table,schema,constraint,statement) \
		IF (OBJECT_ID(REPLACE(QUOTE schema.constraint QUOTE, ' ', '')) IS NOT NULL) \
		BEGIN \
			ALTER TABLE schema.table DROP CONSTRAINT constraint; \
            statement; \
		END
#elif H2
	#define DROP_CONSTRAINT(table,schema,constraint,statement) -- noop
#endif

DROP_CONSTRAINT(EXTENDED_USER_STATES,X4_SCHEMA,FK_EXTENDED_USER_STATES_USER_ID,
    ALTER_TABLE_FK(EXT_USER_STATES_USER_ID,EXTENDED_USER_STATES,USER_ID,USERS,ID,X4_SCHEMA));

DROP_CONSTRAINT(PERMS,X4_SCHEMA,UQ_PERMS_MOD_OBJ_TYPE_ACTION_PARAM,
    ALTER_TABLE_UQ(PERMS_PERMISSION,PERMS,X4_SCHEMA,MODULE,OBJECT_TYPE,ACTION,PARAMETER));

DROP_CONSTRAINT(PERM_TEMPLATES,X4_SCHEMA,UQ_PERM_TEMPLATES_MOD_OBJ_ACT_PAR,
    ALTER_TABLE_UQ(PERM_TEMPLATES_PERMISSION,PERM_TEMPLATES,X4_SCHEMA,OBJECT_TYPE,ACTION,MODULE));

DROP_CONSTRAINT(PASSWORD_POLICIES,X4_SCHEMA,FK_PASSWORD_POLICIES_CREATED_BY,
    ALTER_TABLE_FK(PASSWORD_POLICIES_USER,PASSWORD_POLICIES,CREATED_BY,USERS,ID,X4_SCHEMA));
