#include "include/type.include"
#include "include/constraint.include"
#include "include/schema.include"


ALTER TABLE X4_SCHEMA.X4_CASES ADD CLOSED_AT X4_TYPE_DATETIME_NULLABLE;
ALTER TABLE X4_SCHEMA.X4_CASES ADD PROCESSDEFINITION X4_TYPE_VARCHAR_NULLABLE(255);
ALTER TABLE X4_SCHEMA.X4_CASES ADD TECHNICALSTATE X4_TYPE_VARCHAR_NULLABLE(10);
ALTER TABLE X4_SCHEMA.X4_CASES ADD HOSTNAME X4_TYPE_VARCHAR_NULLABLE(255);

#ifdef MSSQL
GO
#endif

#ifdef MSSQL
	#define X4_DATEDIFF(start, stop) DATEDIFF(ms, start, stop )
	#define X4_DATEADD(millis, start) DATEADD(ms, millis, start )
#elif ORACLE
	#define X4_DATEDIFF(start, stop) (start - stop)
	#define X4_DATEADD(millis, start) start + millis
#else
	#define X4_DATEDIFF(start, stop) EXTRACT(EPOCH FROM (stop - start))
	#define X4_DATEADD(millis, start) start + millis * INTERVAL '1 milliseconds'
#endif

#ifdef ORACLE
UPDATE X4_SCHEMA.X4_CASES 
SET CLOSED_AT = (SELECT 
    	CASE
    		WHEN X4_TECHNICALSTATE = 998 THEN NULL
    		WHEN X4_ENDTIME IS NULL THEN NULL
    		ELSE X4_DATEADD(X4_DATEDIFF(X4_STARTTIME, X4_ENDTIME), CREATED)
    	END
    FROM X4_SCHEMA.PROCESS
    INNER JOIN X4_SCHEMA.X4_OBJECT_TO_PID ON PID = X4_PID
    INNER JOIN X4_SCHEMA.X4_OBJECT_IDS ON OBJECT_UUID = UUID WHERE OBJECT_UUID = X4_CASES.UUID),
    PROCESSDEFINITION = (SELECT 
    	X4_PROCESSDEFINITION
    FROM X4_SCHEMA.PROCESS
    INNER JOIN X4_SCHEMA.X4_OBJECT_TO_PID ON PID = X4_PID
    INNER JOIN X4_SCHEMA.X4_OBJECT_IDS ON OBJECT_UUID = UUID WHERE OBJECT_UUID = X4_CASES.UUID),
    TECHNICALSTATE = (SELECT 
    	CASE 
    		WHEN X4_TECHNICALSTATE = 998 THEN 'WAITING' 
    		WHEN X4_TECHNICALSTATE = -999 THEN 'CRASHED'
    		WHEN X4_TECHNICALSTATE > -1 AND X4_TECHNICALSTATE != 998 THEN 'FINISHED'
    		WHEN X4_TECHNICALSTATE < 0 AND X4_TECHNICALSTATE != -999 THEN 'FAILED'
    		ELSE 'RUNNING'
    	END
    FROM X4_SCHEMA.PROCESS
    INNER JOIN X4_SCHEMA.X4_OBJECT_TO_PID ON PID = X4_PID
    INNER JOIN X4_SCHEMA.X4_OBJECT_IDS ON OBJECT_UUID = UUID WHERE OBJECT_UUID = X4_CASES.UUID),
    HOSTNAME = (SELECT 
    	X4_HOST
    FROM X4_SCHEMA.PROCESS
    INNER JOIN X4_SCHEMA.X4_OBJECT_TO_PID ON PID = X4_PID
    INNER JOIN X4_SCHEMA.X4_OBJECT_IDS ON OBJECT_UUID = UUID WHERE OBJECT_UUID = X4_CASES.UUID);
#elif !H2
UPDATE X4_SCHEMA.X4_CASES 
SET CLOSED_AT = LegacyStructure.X4_ENDTIME,
    PROCESSDEFINITION = LegacyStructure.X4_PROCESSDEFINITION,
    TECHNICALSTATE = LegacyStructure.X4_TECHNICALSTATE,
    HOSTNAME = LegacyStructure.X4_HOST
FROM (
    SELECT OBJECT_UUID, X4_PROCESSDEFINITION, X4_HOST,
    	CASE
    		WHEN X4_TECHNICALSTATE = 998 THEN NULL
    		WHEN X4_ENDTIME IS NULL THEN NULL
    		ELSE X4_DATEADD(X4_DATEDIFF(X4_STARTTIME, X4_ENDTIME), CREATED)
    	END AS X4_ENDTIME,
    	CASE 
    		WHEN X4_TECHNICALSTATE = 998 THEN 'WAITING' 
    		WHEN X4_TECHNICALSTATE = -999 THEN 'CRASHED'
    		WHEN X4_TECHNICALSTATE > -1 AND X4_TECHNICALSTATE != 998 THEN 'FINISHED'
    		WHEN X4_TECHNICALSTATE < 0 AND X4_TECHNICALSTATE != -999 THEN 'FAILED'
    		ELSE 'RUNNING'
    	END AS X4_TECHNICALSTATE
    FROM X4_SCHEMA.PROCESS
    INNER JOIN X4_SCHEMA.X4_OBJECT_TO_PID ON PID = X4_PID
    INNER JOIN X4_SCHEMA.X4_OBJECT_IDS ON OBJECT_UUID = UUID) AS LegacyStructure
WHERE 
    LegacyStructure.OBJECT_UUID = X4_CASES.UUID;
#endif