


-- Table: datalog


CREATE TABLE X4SERVER.DATALOG
(
  x4_pid bigint NOT NULL,
  x4_logtime timestamp  NOT NULL,
  x4_key varchar(50),
  x4_data bytea,
  x4_encoding varchar(20),
  x4_mime_type varchar(100),
  x4_message varchar(255),
  x4_process_id varchar(200) NOT NULL,
  x4_id serial NOT NULL ,
  CONSTRAINT PK_DATALOG PRIMARY KEY (x4_id )
);

GRANT ALL ON X4SERVER.DATALOG TO X4SERVER;


-- Index: "FKI_PROCESS"

-- DROP INDEX "FKI_PROCESS";
CREATE INDEX FKI_PROCESS ON X4SERVER.DATALOG (x4_process_id);

-- Table: process
-- DROP TABLE process;

CREATE TABLE X4SERVER.PROCESS
(
  x4_id varchar(200) NOT NULL,
  x4_parent_id varchar(200),
  x4_pid bigint NOT NULL,
  x4_starttime timestamp  NOT NULL,
  x4_endtime timestamp ,
  x4_user varchar(50) NOT NULL,
  x4_processname varchar(255) NOT NULL,
  x4_version varchar(20) ,
  x4_processdefinition varchar(255) NOT NULL,
  x4_host varchar(100),
  x4_state varchar(100),
  x4_technicalstate integer,
  x4_duration bigint,
  x4_running int,
  CONSTRAINT PK_PROCESS PRIMARY KEY (x4_id )
);

GRANT ALL ON X4SERVER.PROCESS TO X4SERVER;

  -- Index: "IDX_PROCESS_PARENTID"

-- DROP INDEX "IDX_PROCESS_PARENTID";

CREATE INDEX "IDX_PROCESS_PARENTID"
  ON X4SERVER.PROCESS  (x4_parent_id );

-- Index: "IDX_PROCESS_PID"

-- DROP INDEX "IDX_PROCESS_PID";

CREATE INDEX "IDX_PROCESS_PID"
  ON X4SERVER.PROCESS (x4_pid );

-- Index: "IDX_PROCESS_PROCESSNAME"

-- DROP INDEX "IDX_PROCESS_PROCESSNAME";

CREATE INDEX "IDX_PROCESS_PROCESSNAME"
  ON X4SERVER.PROCESS (x4_processname );

-- Index: "IDX_PROCESS_STARTTIME"

-- DROP INDEX "IDX_PROCESS_STARTTIME";

CREATE INDEX "IDX_PROCESS_STARTTIME"
  ON X4SERVER.PROCESS (x4_starttime );

-- Index: "IDX_PROCESS_ENDTIME_TS_ID"

-- DROP INDEX "IDX_PROCESS_ENDTIME_TS_ID";

CREATE INDEX "IDX_PROCESS_ENDTIME_TS_ID"
  ON X4SERVER.PROCESS (x4_endtime desc, x4_technicalstate, x4_id );

-- Index: "IDX_PROCESS_PROCESSDEFINITION"

-- DROP INDEX "IDX_PROCESS_PROCESSDEFINITION";

CREATE INDEX "IDX_PROCESS_PROCESSDEFINITION"
  ON X4SERVER.PROCESS ( X4_PROCESSDEFINITION );

-- Table: process_user

-- DROP TABLE process_user

CREATE TABLE X4SERVER.PROCESS_USER
(
	x4_id serial not null,
	x4_type varchar(255) not null,
	x4_executed_by bigint not null,
	x4_process_id varchar(200) not null,
	CONSTRAINT PK_X4_ID PRIMARY KEY (x4_id),
	CONSTRAINT PROCESS_USER_PROCESS_FK FOREIGN KEY (x4_process_id)
      REFERENCES X4SERVER.PROCESS (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

GRANT ALL ON X4SERVER.PROCESS_USER TO X4SERVER;

-- Table: processhistory

-- DROP TABLE processhistory;

CREATE TABLE X4SERVER.PROCESSHISTORY
(
  x4_pid bigint NOT NULL,
  x4_logtime timestamp NOT NULL,
  x4_user varchar(255),
  x4_technicalstate integer,
  x4_comment varchar(2000),
  x4_process_id varchar(200) NOT NULL,
  x4_host varchar(200) NOT NULL,
  x4_id serial NOT NULL,
  x4_state varchar(50),
  CONSTRAINT PK_PROCESSHISTORY PRIMARY KEY (x4_id )
);

GRANT ALL ON X4SERVER.PROCESSHISTORY TO X4SERVER;


-- Index: "IDX_PROCESSHISTORY_PID"

-- DROP INDEX "IDX_PROCESSHISTORY_PID";

CREATE INDEX "IDX_PROCESSHISTORY_PID"
  ON X4SERVER.PROCESSHISTORY  (x4_pid );

-- Index: "IDX_PROCESSHISTORY_PROCESSID"

-- DROP INDEX "IDX_PROCESSHISTORY_PROCESSID";

CREATE INDEX "IDX_PROCESSHISTORY_PROCESSID"
  ON X4SERVER.PROCESSHISTORY  (x4_process_id );


-- Table: savepoint

-- DROP TABLE savepoint;

CREATE TABLE X4SERVER.SAVEPOINT
(
  x4_pid bigint NOT NULL,
  x4_process_id varchar(200) NOT NULL,
  x4_savetime timestamp NOT NULL,
  x4_data bytea NOT NULL,
  x4_id serial NOT NULL,
  x4_nextactionid integer,
  x4_processdefinition varchar(255), -- URI des Prozesses
  x4_savepoint_id varchar(100) NOT NULL,
  x4_metadata bytea NOT NULL,
  x4_startprocess varchar(255) NOT NULL,
  CONSTRAINT PK_SAVEPOINT PRIMARY KEY (x4_id )
);


GRANT ALL ON X4SERVER.SAVEPOINT TO X4SERVER;

COMMENT ON COLUMN X4SERVER.SAVEPOINT.x4_processdefinition IS 'URI des Prozesses ';

-- Table: sla

-- DROP TABLE sla;


CREATE TABLE X4SERVER.SLA
(
  x4_processdefinition varchar(255) NOT NULL, -- URI of the process
  x4_id serial NOT NULL,
  x4_from varchar(50),
  x4_to varchar(50),
  x4_maxtime bigint NOT NULL,
  x4_workingdays varchar(15) NOT NULL, -- Tag 1-7
  x4_workingtime varchar(10), -- 8:00-16:00
  x4_parameterset varchar(255),
  x4_watchtype integer, -- 1: technisch, 2: fachlich
  x4_escalation_handler varchar(1000),
  CONSTRAINT "PK_SLA" PRIMARY KEY (x4_id )
);
GRANT ALL ON X4SERVER.SLA TO X4SERVER;


COMMENT ON COLUMN X4SERVER.sla.x4_processdefinition IS 'URI of the process';
COMMENT ON COLUMN X4SERVER.sla.x4_workingdays IS 'Tag 1-7';
COMMENT ON COLUMN X4SERVER.sla.x4_workingtime IS '8:00-16:00';
COMMENT ON COLUMN X4SERVER.sla.x4_watchtype IS '1: technisch, 2: fachlich ';

-- Index: "X4Schema"."IDX_SLA_ID"

-- DROP INDEX "X4Schema"."IDX_SLA_ID";

CREATE INDEX "IDX_SLA_ID" ON X4SERVER.SLA  (x4_id );

-- Index: "X4Schema"."IDX_SLA_PROCESSDEFINITION"

-- DROP INDEX "X4Schema"."IDX_SLA_PROCESSDEFINITION";

CREATE INDEX "IDX_SLA_PROCESSDEFINITION"
  ON X4SERVER.SLA  (x4_processdefinition );


-- Table: technicallog

-- DROP TABLE technicallog;

CREATE TABLE X4SERVER.TECHNICALLOG
(
  x4_message varchar(500),
  x4_logtime timestamp NOT NULL,
  x4_pid bigint NOT NULL,
  x4_action integer,
  x4_stepname varchar(255),
  x4_process_id varchar(200) NOT NULL,
  x4_id serial NOT NULL,
  x4_technicalstate integer,
  x4_module varchar(50),
  x4_duration bigint,
  CONSTRAINT PK_TECHNICALLOG PRIMARY KEY (x4_id )
);

GRANT ALL ON X4SERVER.TECHNICALLOG TO X4SERVER;


-- Index: "IDX_TECHNICALLOG_LOGTIME"

-- DROP INDEX "IDX_TECHNICALLOG_LOGTIME";

CREATE INDEX "IDX_TECHNICALLOG_LOGTIME"
  ON X4SERVER.TECHNICALLOG (x4_logtime );

-- Index: "IDX_TECHNICALLOG_PID"

-- DROP INDEX "IDX_TECHNICALLOG_PID";

CREATE INDEX "IDX_TECHNICALLOG_PID"
  ON X4SERVER.TECHNICALLOG  (x4_pid );

-- Index: "IDX_TECHNICALLOG_PROCESSID"

-- DROP INDEX "IDX_TECHNICALLOG_PROCESSID";

CREATE INDEX "IDX_TECHNICALLOG_PROCESSID"
  ON X4SERVER.TECHNICALLOG  (x4_process_id );

-- Index: "IDX_TECHNICALLOG_TECHNICALSTATE"

-- DROP INDEX "IDX_TECHNICALLOG_TECHNICALSTATE";

CREATE INDEX "IDX_TECHNICALLOG_TECHNICALSTATE"
  ON X4SERVER.TECHNICALLOG (x4_technicalstate);


CREATE TABLE X4SERVER.processlock (
    x4_process_id varchar(200) NOT NULL,
    x4_user varchar(100) NOT NULL,
    x4_locktime timestamp NOT NULL,
    x4_id serial NOT NULL,
	CONSTRAINT "PK_LOCK" PRIMARY KEY (x4_id )
);

GRANT ALL ON X4SERVER.PROCESSLOCK TO X4SERVER;

CREATE UNIQUE INDEX LOCK_UNQ_PROCESS_ID ON X4SERVER.PROCESSLOCK(x4_process_id);

-- Table: X4SERVER.JOB

-- DROP TABLE X4SERVER.JOB;

CREATE TABLE X4SERVER.JOB
(
  x4_id serial NOT NULL,
  x4_processdefinition character varying(4000) NOT NULL,
  CONSTRAINT x4_id_pk PRIMARY KEY (x4_id),
  CONSTRAINT x4_processdefinition_unique UNIQUE (x4_processdefinition)
);

GRANT ALL ON X4SERVER.JOB TO X4SERVER;

-- Table: X4SERVER.JOB_PARAMETER

-- DROP TABLE X4SERVER.JOB_PARAMETER;

CREATE TABLE X4SERVER.JOB_PARAMETER
(
  x4_id serial NOT NULL,
  x4_job_id integer NOT NULL,
  x4_key character varying(1024) NOT NULL,
  x4_value character varying(4000) NOT NULL,
  CONSTRAINT job_parameter_pk PRIMARY KEY (x4_id),
  CONSTRAINT job_parameter_fk FOREIGN KEY (x4_job_id)
      REFERENCES X4SERVER.JOB (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

GRANT ALL ON X4SERVER.JOB_PARAMETER TO X4SERVER;

-- Table: X4SERVER.EXECUTION

-- DROP TABLE X4SERVER.EXECUTION;

CREATE TABLE X4SERVER.EXECUTION
(
  x4_id serial NOT NULL,
  x4_type character varying(8) NOT NULL,
  x4_active boolean NOT NULL,
  x4_valid_until timestamp,
  x4_job_id integer NOT NULL,
  x4_valid_from timestamp,
  x4_name character varying(1024) NOT NULL,
  CONSTRAINT execution_pk PRIMARY KEY (x4_id),
  CONSTRAINT execution_job_fk FOREIGN KEY (x4_job_id)
      REFERENCES X4SERVER.JOB (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

GRANT ALL ON X4SERVER.EXECUTION TO X4SERVER;


-- Table: x4server.cron_execution

-- DROP TABLE x4server.cron_execution;

CREATE TABLE x4server.cron_execution
(
  x4_execution_id integer NOT NULL,
  x4_pattern character varying(4000) NOT NULL,
  x4_id serial NOT NULL,
  CONSTRAINT cron_execution_pk PRIMARY KEY (x4_id),
  CONSTRAINT cron_execution_fk FOREIGN KEY (x4_execution_id)
      REFERENCES x4server.execution (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

GRANT ALL ON X4SERVER.CRON_EXECUTION TO X4SERVER;


-- Table: x4server.interval_execution

-- DROP TABLE x4server.interval_execution;

CREATE TABLE x4server.interval_execution
(
  x4_execution_id integer NOT NULL,
  x4_interval integer NOT NULL,
  x4_unit character varying(100) NOT NULL,
  x4_id serial NOT NULL,
  CONSTRAINT interval_execution_pk PRIMARY KEY (x4_id),
  CONSTRAINT interval_execution_fk FOREIGN KEY (x4_execution_id)
      REFERENCES x4server.execution (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

GRANT ALL ON X4SERVER.INTERVAL_EXECUTION To X4SERVER;


-- Table: x4server.execution_parameter

-- DROP TABLE x4server.execution_parameter;

CREATE TABLE X4SERVER.execution_parameter (
  x4_id serial NOT NULL,
  x4_execution_id integer NOT NULL,
  x4_key character varying(1024) NOT NULL,
  x4_value character varying(4000) NOT NULL,
  CONSTRAINT execution_parameter_pk PRIMARY KEY (x4_id),
  CONSTRAINT execution_fk FOREIGN KEY (x4_execution_id)
      REFERENCES x4server.execution (x4_id)
      ON UPDATE NO ACTION ON DELETE NO ACTION
);

GRANT ALL ON X4SERVER.execution_parameter TO X4SERVER;

CREATE TABLE X4SERVER.PLACEHOLDER_GROUPS(
	x4_id serial NOT NULL,
	x4_name character varying(55) NOT NULL,
 	CONSTRAINT PK_PLACEHOLDER_GROUPS PRIMARY KEY (x4_id),
 	CONSTRAINT UK_PLACEHOLDER_GROUPS UNIQUE (x4_name)
);

GRANT ALL ON X4SERVER.PLACEHOLDER_GROUPS TO X4SERVER;

CREATE TABLE X4SERVER.PLACEHOLDER_VALUES(
	x4_id serial NOT NULL,
	x4_name character varying(55) NOT NULL,
	x4_value character varying(255) NULL,
	x4_type integer NULL,
	x4_group_id integer NOT NULL,
 	CONSTRAINT PK_PLACEHOLDER_VALUES PRIMARY KEY (x4_id),
	CONSTRAINT UK_PLACEHOLDER_VALUES UNIQUE (x4_name, x4_group_id),
 	CONSTRAINT FK_PLACEHOLDER_VALUES_PLACEHOLDER_GROUPS FOREIGN KEY (x4_group_id)
      REFERENCES X4SERVER.PLACEHOLDER_GROUPS (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

GRANT ALL ON X4SERVER.PLACEHOLDER_VALUES TO X4SERVER;


-- Dev Day Tables
GRANT ALL ON X4SERVER.PLACEHOLDER_VALUES TO X4SERVER;

CREATE TABLE x4server.time_project
(
    id varchar(100) NOT NULL,
    name varchar(100),
    CONSTRAINT time_project_pkey PRIMARY KEY (id)
);

GRANT ALL ON x4server.time_project TO x4server;



CREATE TABLE x4server.time_tracking
(
    id serial NOT NULL,
    date timestamp,
    project varchar(100),
    duration double,
    note varchar(1000),
    warranty boolean,
    CONSTRAINT time_tracking_pkey PRIMARY KEY (id)
);

GRANT ALL ON x4server.time_tracking TO x4server;


-- Table: x4server.webapp_rights

-- DROP TABLE x4server.webapp_rights;

CREATE TABLE x4server.webapp_rights
(
  id serial NOT NULL,
  type varchar(10) NOT NULL,
  usergroupid integer NOT NULL,
  webapp_project_id UUID NOT NULL,
  CONSTRAINT pk_webapp_rights_id PRIMARY KEY (id)
);

GRANT ALL ON x4server.webapp_rights TO x4server;

