CREATE TABLE X4SERVER.DATALOG
(
  x4_pid bigint NOT NULL,
  x4_logtime datetime  NOT NULL,
  x4_key nvarchar(50),
  x4_data varbinary(max),
  x4_encoding nvarchar(20),
  x4_mime_type nvarchar(100),
  x4_message nvarchar(255),
  x4_process_id nvarchar(200) NOT NULL,
  x4_id bigint IDENTITY(1,1) NOT NULL ,
  CONSTRAINT PK_DATALOG PRIMARY KEY (x4_id )
);

-- Index: "FKI_PROCESS"

-- DROP INDEX "FKI_PROCESS";
CREATE INDEX FKI_PROCESS ON X4SERVER.DATALOG (x4_process_id);

-- Table: process
-- DROP TABLE process;

CREATE TABLE X4SERVER.PROCESS
(
  x4_id nvarchar(200) NOT NULL,
  x4_parent_id nvarchar(200),
  x4_pid bigint NOT NULL,
  x4_starttime datetime  NOT NULL,
  x4_endtime datetime ,
  x4_user nvarchar(50) NOT NULL,
  x4_processname nvarchar(255) NOT NULL,
  x4_version nvarchar(20) ,
  x4_processdefinition nvarchar(255) NOT NULL,
  x4_host nvarchar(100),
  x4_state nvarchar(100),
  x4_technicalstate integer,
  x4_duration bigint,
  x4_running int,
  CONSTRAINT PK_PROCESS PRIMARY KEY (x4_id )
);

CREATE INDEX IDX_PROCESS_PARENTID
  ON X4SERVER.PROCESS  (x4_parent_id );

CREATE INDEX IDX_PROCESS_PID
  ON X4SERVER.PROCESS (x4_pid );

CREATE INDEX IDX_PROCESS_PROCESSNAME
  ON X4SERVER.PROCESS (x4_processname );

CREATE INDEX IDX_PROCESS_STARTTIME
  ON X4SERVER.PROCESS (x4_starttime );

CREATE INDEX IDX_PROCESS_ENDTIME_TS_ID
  ON X4SERVER.PROCESS (x4_endtime desc, x4_technicalstate, x4_id );

CREATE INDEX IDX_PROCESS_USER_ENDTIME
  ON X4SERVER.PROCESS (x4_user,x4_endtime )
  INCLUDE (x4_id)


CREATE INDEX IDX_PROCESS_PROCESSDEFINITION
  ON X4SERVER.PROCESS (x4_processdefinition );

CREATE INDEX IDX_PROCESS_RUNNING
  ON X4SERVER.PROCESS (x4_running );

-- Table: process_user

-- DROP TABLE process_user

CREATE TABLE X4SERVER.PROCESS_USER
(
	x4_id bigint IDENTITY(1,1) not null,
	x4_type nvarchar(255) not null,
	x4_executed_by bigint not null,
	x4_process_id nvarchar(200) not null,
	CONSTRAINT PK_X4_ID PRIMARY KEY (x4_id),
	CONSTRAINT PROCESS_USER_PROCESS_FK FOREIGN KEY (x4_process_id)
      REFERENCES X4SERVER.PROCESS (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

-- Table: processhistory

-- DROP TABLE processhistory;

CREATE TABLE X4SERVER.PROCESSHISTORY
(
  x4_pid bigint NOT NULL,
  x4_logtime datetime NOT NULL,
  x4_user nvarchar(255),
  x4_technicalstate integer,
  x4_comment nvarchar(2000),
  x4_process_id nvarchar(200) NOT NULL,
  x4_host nvarchar(200) NOT NULL,
  x4_id bigint IDENTITY(1,1) NOT NULL,
  x4_state nvarchar(50),
  CONSTRAINT PK_PROCESSHISTORY PRIMARY KEY (x4_id )
);

-- Index: "IDX_PROCESSHISTORY_PID"

-- DROP INDEX "IDX_PROCESSHISTORY_PID";

CREATE INDEX IDX_PROCESSHISTORY_PID
  ON X4SERVER.PROCESSHISTORY  (x4_pid );
-- Index: "IDX_PROCESSHISTORY_PROCESSID"

-- DROP INDEX "IDX_PROCESSHISTORY_PROCESSID";

CREATE INDEX IDX_PROCESSHISTORY_PROCESSID
  ON X4SERVER.PROCESSHISTORY  (x4_process_id );

-- Table: savepoint

-- DROP TABLE savepoint;

CREATE TABLE X4SERVER.SAVEPOINT
(
  x4_pid bigint NOT NULL,
  x4_process_id nvarchar(200) NOT NULL,
  x4_savetime datetime NOT NULL,
  x4_data varbinary(max) NOT NULL,
  x4_id bigint IDENTITY(1,1) NOT NULL,
  x4_nextactionid integer,
  x4_processdefinition nvarchar(255), -- URI des Prozesses
  x4_savepoint_id nvarchar(100) NOT NULL,
  x4_metadata varbinary(8000) NOT NULL,
  x4_startprocess nvarchar(255) NOT NULL,
  CONSTRAINT PK_SAVEPOINT PRIMARY KEY (x4_id )
);

-- COMMENT ON COLUMN X4SERVER.SAVEPOINT.x4_processdefinition IS 'URI des Prozesses ';

-- Table: sla

-- DROP TABLE sla;


CREATE TABLE X4SERVER.SLA
(
  x4_processdefinition nvarchar(255) NOT NULL, -- URI of the process
  x4_id bigint IDENTITY(1,1) NOT NULL,
  x4_from nvarchar(50),
  x4_to nvarchar(50),
  x4_maxtime bigint NOT NULL,
  x4_workingdays nvarchar(15) NOT NULL, -- Tag 1-7
  x4_workingtime nvarchar(10), -- 8:00-16:00
  x4_parameterset nvarchar(255),
  x4_watchtype integer, -- 1: technisch, 2: fachlich
  x4_escalation_handler nvarchar(1000),
  CONSTRAINT PK_SLA PRIMARY KEY (x4_id )
);

-- COMMENT ON COLUMN X4SERVER.SLA.x4_processdefinition IS 'URI of the process';
-- COMMENT ON COLUMN X4SERVER.SLA.x4_workingdays IS 'Tag 1-7';
-- COMMENT ON COLUMN X4SERVER.SLA.x4_workingtime IS '8:00-16:00';
-- COMMENT ON COLUMN X4SERVER.SLA.x4_watchtype IS '1: technisch, 2: fachlich ';
-- Index: "X4Schema"."IDX_SLA_ID"

-- DROP INDEX "X4Schema"."IDX_SLA_ID";

CREATE INDEX IDX_SLA_ID ON X4SERVER.SLA  (x4_id );
-- Index: "X4Schema"."IDX_SLA_PROCESSDEFINITION"

-- DROP INDEX "X4Schema"."IDX_SLA_PROCESSDEFINITION";

CREATE INDEX IDX_SLA_PROCESSDEFINITION
  ON X4SERVER.SLA  (x4_processdefinition );

-- Table: technicallog

-- DROP TABLE technicallog;

CREATE TABLE X4SERVER.TECHNICALLOG
(
  x4_message nvarchar(500),
  x4_logtime datetime NOT NULL,
  x4_pid bigint NOT NULL,
  x4_action integer,
  x4_stepname nvarchar(255),
  x4_process_id nvarchar(200) NOT NULL,
  x4_id bigint IDENTITY(1,1) NOT NULL,
  x4_technicalstate integer,
  x4_module nvarchar(50),
  x4_duration bigint,
  CONSTRAINT PK_TECHNICALLOG PRIMARY KEY (x4_id )
);

-- Index: "IDX_TECHNICALLOG_LOGTIME"

-- DROP INDEX "IDX_TECHNICALLOG_LOGTIME";

CREATE INDEX IDX_TECHNICALLOG_LOGTIME
  ON X4SERVER.TECHNICALLOG (x4_logtime );
-- Index: "IDX_TECHNICALLOG_PID"

-- DROP INDEX "IDX_TECHNICALLOG_PID";

CREATE INDEX IDX_TECHNICALLOG_PID
  ON X4SERVER.TECHNICALLOG  (x4_pid );
-- Index: "IDX_TECHNICALLOG_PROCESSID"

-- DROP INDEX "IDX_TECHNICALLOG_PROCESSID";

CREATE INDEX IDX_TECHNICALLOG_PROCESSID
  ON X4SERVER.TECHNICALLOG  (x4_process_id );

-- Index: "IDX_TECHNICALLOG_TECHNICALSTATE"

-- DROP INDEX "IDX_TECHNICALLOG_TECHNICALSTATE";

CREATE INDEX IDX_TECHNICALLOG_TECHNICALSTATE
  ON X4SERVER.TECHNICALLOG (x4_technicalstate);

CREATE TABLE X4SERVER.PROCESSLOCK (
    x4_process_id nvarchar(200) NOT NULL,
    x4_user nvarchar(100) NOT NULL,
    x4_locktime datetime NOT NULL,
    x4_id bigint IDENTITY(1,1) NOT NULL,
	CONSTRAINT PK_LOCK PRIMARY KEY (x4_id )
);

CREATE UNIQUE INDEX LOCK_UNQ_PROCESS_ID ON X4SERVER.PROCESSLOCK(x4_process_id);

CREATE TABLE X4SERVER.JOB (
	x4_id bigint IDENTITY(1,1) NOT NULL,
	x4_processdefinition nvarchar(450) NOT NULL,
	CONSTRAINT X4_ID_PK PRIMARY key (x4_id),
	CONSTRAINT X4_PROCESSDEFINITION_UNIQUE UNIQUE (x4_processdefinition)
);

CREATE TABLE X4SERVER.JOB_PARAMETER (
	x4_id bigint IDENTITY(1,1) NOT NULL,
	x4_job_id bigint NOT NULL,
	x4_key nvarchar(1024) NOT NULL,
	x4_value nvarchar(4000) NOT NULL,
	CONSTRAINT JOB_PARAMETER_PK PRIMARY KEY (x4_id),
	CONSTRAINT JOB_PARAMETER_FK FOREIGN KEY (x4_job_id) REFERENCES X4SERVER.JOB (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

CREATE TABLE X4SERVER.EXECUTION(
	x4_id bigint IDENTITY(1,1) NOT NULL,
	x4_type nvarchar(8) NOT NULL,
	x4_active bigint NOT NULL,
	x4_valid_until datetime,
	x4_job_id bigint NOT NULL,
	x4_valid_from datetime,
	x4_name nvarchar(1024) NOT NULL,
	CONSTRAINT EXECUTION_PK PRIMARY KEY (x4_id),
	CONSTRAINT EXECUTION_JOB_FK FOREIGN KEY (x4_job_id) REFERENCES X4SERVER.JOB (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION,
	CONSTRAINT ACTIVE_CHECK CHECK (x4_active IN (0, 1))
);

CREATE TABLE X4SERVER.CRON_EXECUTION(
	x4_execution_id bigint NOT NULL,
	x4_pattern nvarchar(4000) NOT NULL,
	x4_id bigint IDENTITY(1,1) NOT NULL,
	CONSTRAINT CRON_EXECUTION_PK PRIMARY KEY (x4_id),
	CONSTRAINT CRON_EXECUTION_FK FOREIGN KEY (x4_execution_id) REFERENCES x4server.execution (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

CREATE TABLE X4SERVER.INTERVAL_EXECUTION(
	x4_execution_id bigint NOT NULL,
	x4_interval bigint NOT NULL,
	x4_unit nvarchar(100) NOT NULL,
	x4_id bigint IDENTITY(1,1) NOT NULL,
	CONSTRAINT INTERVAL_EXECUTION_PK PRIMARY KEY (x4_id),
	CONSTRAINT INTERVAL_EXECUTION_FK FOREIGN KEY (x4_execution_id) REFERENCES x4server.execution (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

CREATE TABLE X4SERVER.EXECUTION_PARAMETER(
	x4_id bigint IDENTITY(1,1) NOT NULL,
	x4_execution_id bigint NOT NULL,
	x4_key nvarchar(1024) NOT NULL,
	x4_value nvarchar(4000) NOT NULL,
	CONSTRAINT EXECUTION_PARAMETER_PK PRIMARY KEY (x4_id),
	CONSTRAINT EXECUTION_PARAMETER_FK FOREIGN KEY (x4_execution_id) REFERENCES x4server.execution (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

CREATE TABLE X4SERVER.PLACEHOLDER_GROUPS(
	x4_id bigint IDENTITY(1,1) NOT NULL,
	x4_name nvarchar(55) NOT NULL,
 	CONSTRAINT PK_PLACEHOLDER_GROUPS PRIMARY KEY (x4_id ASC),
 	CONSTRAINT UK_PLACEHOLDER_GROUPS UNIQUE (x4_name)
);

CREATE TABLE X4SERVER.PLACEHOLDER_VALUES(
	x4_id bigint IDENTITY(1,1) NOT NULL,
	x4_name nvarchar(55) NOT NULL,
	x4_value nvarchar(255) NULL,
	x4_type tinyint NULL,
	x4_group_id bigint NOT NULL,
 	CONSTRAINT PK_PLACEHOLDER_VALUES PRIMARY KEY (x4_id ASC),
	CONSTRAINT UK_PLACEHOLDER_VALUES UNIQUE (x4_name, x4_group_id),
 	CONSTRAINT FK_PLACEHOLDER_VALUES_PLACEHOLDER_GROUPS FOREIGN KEY (x4_group_id)
      REFERENCES X4SERVER.PLACEHOLDER_GROUPS (x4_id) ON UPDATE NO ACTION ON DELETE NO ACTION
);

-- Table: x4server.webapp_rights

-- DROP TABLE x4server.webapp_rights;

CREATE TABLE X4SERVER.WEBAPP_RIGHTS
(
  id bigint IDENTITY(1,1) NOT NULL,
  type nvarchar(10) NOT NULL,
  usergroupid bigint NOT NULL,
  webapp_project_id UNIQUEIDENTIFIER NOT NULL,
  CONSTRAINT pk_webapp_rights_id PRIMARY KEY (id)
);
