<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
	<xsl:output media-type="text/xml" method="xml" indent="yes" />
	<xsl:template match="/*" >
		<xsl:copy-of select="." />
	</xsl:template>
	<xsl:template match="/Route" priority="1">
		<xsl:element name="adapter">
			<xsl:apply-templates select="@*" />
			<xsl:call-template name="fixNameAttribute">
				<xsl:with-param name="name" select="@name" />
				<xsl:with-param name="code" select="@code" />
			</xsl:call-template>
			<states />
			<xsl:apply-templates select="actions" />
			<properties>
				<xsl:apply-templates select="Parameter" />
			</properties>
		</xsl:element>
	</xsl:template>
	<xsl:template match="/FunctionAdapter" priority="1">
		<xsl:element name="adapter">
			<xsl:copy-of select="@*" />
			<xsl:call-template name="fixNameAttribute">
				<xsl:with-param name="name" select="@name" />
				<xsl:with-param name="code" select="@code" />
			</xsl:call-template>
			<xsl:copy-of select="states" />
			<xsl:apply-templates select="actions" />
			<properties>
				<xsl:apply-templates select="Parameter" />
			</properties>
		</xsl:element>
	</xsl:template>
	<xsl:template match="Parameter">
		<property basic_property="true" optional="true">
			<xsl:apply-templates select="@*" />
		</property>
	</xsl:template>
	<xsl:template match="action/name">
		<xsl:copy>
			<xsl:choose>
				<xsl:when test="contains(.,'|')">
					<xsl:value-of select="substring-after(., '|')" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="." />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:copy>
	</xsl:template>
	<xsl:template match="@iXname">
		<xsl:attribute name="iXname">
			<xsl:choose>
				<xsl:when test="contains(.,'|')">
					<xsl:value-of select="substring-after(., '|')" />
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="." />
				</xsl:otherwise>
			</xsl:choose>
		</xsl:attribute>
	</xsl:template>
	<xsl:template match="@optional">
		<xsl:attribute name="optional">
			<xsl:choose>
				<xsl:when test=". = 'yes' or . = 'true'">
					<xsl:text>true</xsl:text>
				</xsl:when>
				<xsl:otherwise>
					<xsl:text>false</xsl:text>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:attribute>
	</xsl:template>
	<xsl:template match="@*|text()|node()">
		<xsl:copy>
			<xsl:apply-templates select="@*" />
			<xsl:apply-templates
				select="child::node()|text()" />
		</xsl:copy>
	</xsl:template>
	<xsl:template name="fixNameAttribute">
		<xsl:param name="name" />
		<xsl:param name="code" />
		<xsl:if test="normalize-space($name) = ''">
			<xsl:attribute name="name">
				<xsl:choose>
					<xsl:when test="$code = 'route_001'">FTP Transfer</xsl:when>
					<xsl:when test="$code = 'route_002'">POP/SMTP</xsl:when>
					<xsl:when test="$code = 'route_003'">HTTP Transfer</xsl:when>
					<xsl:when test="$code = 'route_005'">JMS</xsl:when>
					<xsl:when test="$code = 'route_006'">File System</xsl:when>
					<xsl:when test="$code = 'route_007'">SMS</xsl:when>
					<xsl:when test="$code = 'route_008'">SFTP Transfer</xsl:when>
					<xsl:when test="$code = 'route_010'">RequestReply Transfer</xsl:when>
					<xsl:when test="$code = 'route_011'">MQ Series Transfer</xsl:when>
					<xsl:when test="$code = 'ElementWrapperAdapter'">de.softproject.integration.adapter.elementwrapper.ElementWrapperAdapter</xsl:when>
				 	<xsl:when test="$code = 'FA_000021'">de.softproject.integration.adapter.log4j.Log4jAdapter</xsl:when>
				 	<xsl:when test="$code = 'DocumentPropertiesAdapter'">de.softproject.integration.adapter.documentproperties.DocumentPropertiesAdapter</xsl:when>
				 	<xsl:when test="$code = 'StreamSplitter'">de.softproject.integration.adapter.streamsplitter.StreamSplitter</xsl:when>
				  	<xsl:when test="$code = 'FA_000022'">de.softproject.integration.adapter.processstarter.ProcessStarter</xsl:when>
					<xsl:when test="$code = 'XPathSelector'">de.softproject.integration.adapter.xpathselector.XPathSelector</xsl:when>
					<xsl:when test="$code = 'XMLFormatter'">de.softproject.integration.adapter.xmlformatter.XMLFormatterAdapter</xsl:when>
					<xsl:when test="$code = 'XMLValidator'">de.softproject.integration.adapter.xmlvalidator.XMLValidator</xsl:when>
					<xsl:when test="$code = 'ChecksumAdapter'">de.softproject.integration.adapter.checksum.ChecksumAdapter</xsl:when>
					<xsl:when test="$code = 'Encoding Converter'">de.softproject.integration.adapter.encodingconversion.DocumentEncodingAdapter</xsl:when>
					<xsl:when test="$code = 'FA_000002'">de.softproject.integration.adapter.asciireader.Adapter</xsl:when>
					<xsl:when test="$code = 'FA_000004'">de.softproject.integration.adapter.ASCIIGDV</xsl:when>
					<xsl:when test="$code = 'FA_000005'">de.softproject.integration.adapter.GDVWriter</xsl:when>
					<xsl:when test="$code = 'FA_000006'">de.softproject.integration.adapter.zipadapter.ZipAdapter</xsl:when>
					<xsl:when test="$code = 'FA_000007'">de.softproject.integration.adapter.docadapter.DocAdapter</xsl:when>
					<xsl:when test="$code = 'FA_000010'">de.softproject.integration.adapter.cmdadapter.CMDAdapter</xsl:when>
					<xsl:when test="$code = 'FA_000011'">de.softproject.integration.adapter.osprocess.OsProcessAdapter</xsl:when>
					<xsl:when test="$code = 'FA_000014'">de.softproject.integration.adapter.CopyBookReader</xsl:when>
					<xsl:when test="$code = 'FA_000015'">de.softproject.integration.adapter.CopyBookWriter</xsl:when>
					<xsl:when test="$code = 'FA_000016'">de.softproject.integration.adapter.gdvplausi.gdvPlausiPrueferWrapper</xsl:when>
					<xsl:when test="$code = 'FA_000020'">de.softproject.integration.adapter.PropertiesReader</xsl:when>
					<xsl:when test="$code = 'FA_000023'">de.softproject.integration.adapter.pcmladapter.PCMLAdapter</xsl:when>
					<xsl:when test="$code = 'FA_000026'">de.softproject.integration.adapter.zipfolderadapter.ZipFolderAdapter</xsl:when>
					<xsl:when test="$code = 'HTTPAdapter'">de.softproject.integration.adapter.httpadapter.HttpAdapter</xsl:when>
					<xsl:when test="$code = 'XMLValidatorNew'">de.softproject.integration.adapter.xmlvalidator.XMLValidator</xsl:when>
					<xsl:when test="$code = 'nyi.Sharepoint'">nyi.Sharepoint</xsl:when>
					<xsl:when test="$code = 'nyi.Dynamics'">nyi.Dynamics</xsl:when>
					<xsl:when test="$code = 'FA_000008'">de.softproject.integration.adapter.wordheaderadapter.wordHeaderAdapter</xsl:when>
					<xsl:when test="$code = 'Word2XML'">de.softproject.integration.adapter.Word2XML</xsl:when>
					<xsl:when test="$code = 'SAPAdapter'">de.softproject.x4.sapadapter.SAPAdapter</xsl:when>
					<xsl:when test="$code = 'de.softproject.integration.adapter.xml.encryption.XmlEncryption.Password'">de.softproject.integration.adapter.xml.encryption.XmlEncryption.Password</xsl:when>
					<xsl:when test="$code = 'de.softproject.integration.adapter.xml.encryption.XmlEncryption.KeyStore'">de.softproject.integration.adapter.xml.encryption.XmlEncryption.KeyStore</xsl:when>
					<xsl:when test="$code = 'FA_000003'">de.softproject.integration.adapter.asciiwriter.ASCIIWriter</xsl:when>
					<xsl:otherwise><xsl:value-of select="$code" /></xsl:otherwise>
				</xsl:choose>
			</xsl:attribute>
		</xsl:if>
	</xsl:template>
</xsl:stylesheet>
<!--Created by X4 Designer, Copyright © SoftProject GmbH. All rights reserved. -->